import argparse
import logging

from tutumcli import parsers
from tutumcli import commands


VERSION = "0.6.6"


logging.basicConfig()

# Main parser
parser = argparse.ArgumentParser(description="Tutum's CLI", prog="tutum")
parser.add_argument('-v', '--version', action='version', version='%(prog)s ' + VERSION)

subparsers = parser.add_subparsers(title="Tutum's CLI commands", dest='command')

# Common options
parent_parser = argparse.ArgumentParser(add_help=False)

# Commands
parsers.add_login_parser(subparsers, parent_parser)
parsers.add_apps_parser(subparsers, parent_parser)
parsers.add_containers_parser(subparsers, parent_parser)


def main():
    # Parse args
    args = parser.parse_args()
    if args.command == "login":
        commands.authenticate()
    elif args.command == "apps":
        commands.apps()
    elif args.command == "inspect":
        commands.application_details(args.identifier)
    elif args.command == "start":
        commands.app_start(args.identifier)
    elif args.command == "stop":
        commands.app_stop(args.identifier)
    elif args.command == "terminate":
        commands.app_terminate(args.identifier)
    elif args.command == "logs":
        commands.app_logs(args.identifier)
    elif args.command == "scale":
        commands.app_scale(args.identifier, args.target_num_containers)
    elif args.command == "alias":
        commands.app_alias(args.identifier, args.dns)
    elif args.command == "run":
        commands.app_run(image=args.image, name=args.name, container_size=args.container_size,
                         target_num_containers=args.target_num_containers, run_command=args.run_command,
                         entrypoint=args.entrypoint, container_ports=args.port,
                         container_envvars=args.env,
                         linked_to_application=args.link, autorestart=args.autorestart,
                         autoreplace=args.autoreplace, autodestroy=args.autodestroy, roles=args.role)
    elif args.command == "ps":
            commands.ps(args.identifier)
    elif args.command == "inspect-container":
        print "inspect"
        commands.container_inspect(args.identifier)
    elif args.command == "logs-container":
        commands.container_logs(args.identifier)
    elif args.command == "start-container":
        commands.container_start(args.identifier)
    elif args.command == "stop-container":
        commands.container_stop(args.identifier)
    elif args.command == "terminate-container":
        commands.container_terminate(args.identifier)

if __name__ == "__main__":
    main()


import getpass
import ConfigParser
from os.path import join, expanduser
import pprint

from tutum.api import auth
from tutum.api import exceptions

import tutum
from tutumcli import utils


TUTUM_FILE = '.tutum'
AUTH_SECTION = 'auth'
USER_OPTION = "user"
APIKEY_OPTION = 'apikey'


def authenticate():

    username = raw_input("Username: ")
    password = getpass.getpass()
    try:
        api_key = auth.get_apikey(username, password)
        if api_key is not None:
            config = ConfigParser.ConfigParser()
            config.add_section(AUTH_SECTION)
            config.set(AUTH_SECTION, USER_OPTION, username)
            config.set(AUTH_SECTION, APIKEY_OPTION, api_key)
            with open(join(expanduser('~'), TUTUM_FILE), 'w') as cfgfile:
                config.write(cfgfile)
            print "Login succeeded!"
    except exceptions.TutumAuthError:
        print "Wrong username and/or password. Please try to login again"
    except Exception as e:
        print e


def apps():
    try:
        app_list = tutum.Application.list()
        headers = ["Name", "UUID", "State", "Image", "Size", "Deployed datetime", "Web Hostname"]
        data_list = []
        if len(app_list) != 0:
            for app in app_list:
                data_list.append([app.unique_name, app.uuid[:8], app.state, app.image_tag, app.container_size,
                                  app.deployed_datetime, app.web_public_dns])
        else:
            data_list.append(["", "", "", "", "", "", ""])
        utils.tabulate_result(data_list, headers)
    except Exception as e:
        print e


def application_details(identifier):
    try:
        app_details = utils.fetch_app(identifier)
        pprint.pprint(app_details.get_all_attributes())
    except Exception as e:
        print e


def app_start(identifier):
    try:
        app_details = utils.fetch_app(identifier)
        result = app_details.start()
        if result:
            print app_details.uuid
    except Exception as e:
        print e


def app_stop(identifier):
    try:
        app_details = utils.fetch_app(identifier)
        result = app_details.stop()
        if result:
            print app_details.uuid
    except Exception as e:
        print e


def app_terminate(identifier):
    try:
        app_details = utils.fetch_app(identifier)
        result = app_details.delete()
        if result:
            print app_details.uuid
    except Exception as e:
        print e


def app_logs(identifier):
    try:
        app_details = utils.fetch_app(identifier)
        print app_details.logs
    except Exception as e:
        print e


def app_scale(identifier, target_num_containers):
    try:
        app_details = utils.fetch_app(identifier)
        if target_num_containers:
            app_details.target_num_containers = target_num_containers
            result = app_details.save()
            if result:
                print app_details.uuid
    except Exception as e:
        print e


def app_alias(identifier, dns):
    try:
        app_details = utils.fetch_app(identifier)
        if dns:
            app_details.web_public_dns = dns
            result = app_details.save()
            if result:
                print app_details.uuid
    except Exception as e:
        print e


def app_run(image, name, container_size, target_num_containers, run_command, entrypoint, container_ports,
            container_envvars, linked_to_application, autorestart, autoreplace, autodestroy, roles):
    try:
        ports = utils.parse_ports(container_ports)
        envvars = utils.parse_envvars(container_envvars)
        app = tutum.Application.create(image=image, name=name, container_size=container_size,
                                       target_num_containers=target_num_containers, run_command=run_command,
                                       entrypoint=entrypoint, container_ports=ports,
                                       container_envvars=envvars, linked_to_application=linked_to_application,
                                       autorestart=autorestart, autoreplace=autoreplace, autodestroy=autodestroy, roles=roles)
        result = app.save()
        if result:
            print app.uuid
    except Exception as e:
        print e


def ps(app_identifier):
    try:
        if app_identifier is None:
            containers = tutum.Container.list()
        elif utils.is_uuid4(app_identifier):
            containers = tutum.Container.list(application__uuid=app_identifier)
        else:
            containers = tutum.Container.list(application__name=app_identifier) + tutum.Container.list(application__uuid__startswith=app_identifier)
        headers = ["Name", "UUID", "State", "Image", "Run Command", "Size", "Exit Code", "Deployed datetime", "Ports"]
        data_list = []
        if len(containers) != 0:
            for container in containers:
                ports_string = ""
                for index, port in enumerate(container.container_ports):
                    if port['outer_port'] is not None:
                        ports_string += "%s:%d->" % (container.public_dns, port['outer_port'])
                    ports_string += "%d/%s" % (port['inner_port'], port['protocol'])
                    if index != len(container.container_ports) - 1:
                        ports_string += ", "
                data_list.append([container.unique_name, container.uuid[:8], container.state, container.image_tag,
                                  container.run_command, container.container_size, container.exit_code,
                                  container.deployed_datetime, ports_string])
        else:
            data_list.append(["", "", "", "", "", "", "", "", ""])
        utils.tabulate_result(data_list, headers)
    except Exception as e:
        print e


def container_inspect(identifier):
    try:
        container_details = utils.fetch_container(identifier)
        pprint.pprint(container_details.get_all_attributes())
    except Exception as e:
        print e


def container_start(identifier):
    try:
        container_details = utils.fetch_container(identifier)
        result = container_details.start()
        if result:
            print container_details.uuid
    except Exception as e:
        print e


def container_stop(identifier):
    try:
        container_details = utils.fetch_container(identifier)
        result = container_details.stop()
        if result:
            print container_details.uuid
    except Exception as e:
        print e


def container_terminate(identifier):
    try:
        container_details = utils.fetch_container(identifier)
        result = container_details.delete()
        if result:
            print container_details.uuid
    except Exception as e:
        print e


def container_logs(identifier):
    try:
        container_details = utils.fetch_container(identifier)
        print container_details.logs
    except Exception as e:
        print e


import argparse


def add_login_parser(subparsers, parent_parser):
    login_parser = subparsers.add_parser('login', help='Login into Tutum', parents=[parent_parser])
    return login_parser


def add_apps_parser(subparsers, parent_parser):
    apps_parser = subparsers.add_parser('apps', help='List all applications', parents=[parent_parser])

    # App common options
    app_common_parser = argparse.ArgumentParser(add_help=False)
    app_common_parser.add_argument("identifier", help="Application's uuid (either long or short) or name")

    create_app_parser = subparsers.add_parser('run', help='Create and run an application', parents=[parent_parser])
    create_app_parser.add_argument("image", help="the image used to deploy this application in docker format")
    create_app_parser.add_argument("-n", "--name", help="a human-readable name for the application "
                                                        "(default: image_tag without namespace)")
    create_app_parser.add_argument("-s", "--container_size", help="the size of the application containers "
                                                                  "(default: XS, possible values: XS, S, M, L, XL)")
    create_app_parser.add_argument("-t", "--target_num_containers", help="the number of containers to run "
                                                                         "for this application (default: 1)", type=int)
    create_app_parser.add_argument("-r", "--run_command",
                                   help="the command used to start the application containers "
                                        "(default: as defined in the image)")
    create_app_parser.add_argument("--entrypoint",
                                   help="the command prefix used to start the application containers "
                                        "(default: as defined in the image)")
    create_app_parser.add_argument("-p", "--port",
                                   help="set ports i.e. '80/tcp'"
                                        "(default: as defined in the image)", action='append')
    create_app_parser.add_argument("-e", "--env",
                                   help="set environment variables i.e. 'ENVVAR=foo' "
                                        "(default: as defined in the image, plus any link- or role-generated variables)",
                                   action='append')
    create_app_parser.add_argument("-l", "--link",
                                   help="an application's uuid (either long or short) or name to link this application "
                                        "to, i.e. 80ff1635-2d56-478d-a97f-9b59c720e513'] (default: none)",
                                   action='append')
    create_app_parser.add_argument('--autorestart', help="whether the containers should be restarted if they stop "
                                                         "(default: OFF)", choices=['OFF', 'ON_FAILURE', 'ALWAYS'])
    create_app_parser.add_argument('--autoreplace', help="whether the containers should be replaced with a new one if "
                                                         "they stop (default: OFF)",
                                   choices=['OFF', 'ON_FAILURE', 'ALWAYS'])
    create_app_parser.add_argument('--autodestroy', help="whether the containers should be terminated if "
                                                         "they stop (default: OFF)",
                                   choices=['OFF', 'ON_FAILURE', 'ALWAYS'])
    create_app_parser.add_argument('--role', help="Tutum API roles to grant the application, "
                                                  "i.e. 'global' (default: none, possible values: 'global')",
                                   action='append')

    inspect_app_parser = subparsers.add_parser('inspect', help='Inspect an application',
                                               parents=[parent_parser, app_common_parser])
    start_app_parser = subparsers.add_parser('start', help='Start an application',
                                             parents=[parent_parser, app_common_parser])
    stop_app_parser = subparsers.add_parser('stop', help='Stop an application',
                                            parents=[parent_parser, app_common_parser])
    terminate_app_parser = subparsers.add_parser('terminate', help='Terminate an application',
                                                 parents=[parent_parser, app_common_parser])
    logs_app_parser = subparsers.add_parser('logs', help='Get logs from an application',
                                            parents=[parent_parser, app_common_parser])
    scale_app_parser = subparsers.add_parser('scale', help='Scale an application',
                                             parents=[parent_parser, app_common_parser])
    scale_app_parser.add_argument("target_num_containers",
                                  help="target number of containers to scale this application to",
                                  type=int)

    alias_app_parser = subparsers.add_parser('alias', help="Change application's dns",
                                             parents=[parent_parser, app_common_parser])
    alias_app_parser.add_argument("dns", help="custom domain to use for this web application")

    return apps_parser


def add_containers_parser(subparsers, parent_parser):
    containers_parser = subparsers.add_parser('ps', help='List all containers', parents=[parent_parser])
    containers_parser.add_argument("-i", "--identifier", help="Application's uuid (either long or short) or name")

    # Container common options
    container_common_parser = argparse.ArgumentParser(add_help=False)
    container_common_parser.add_argument("identifier", help="Container's uuid (either long or short) or name")

    inspect_container_parser = subparsers.add_parser('inspect-container', help='Inspect a container',
                                                     parents=[parent_parser, container_common_parser])
    start_container_parser = subparsers.add_parser('start-container', help='Start a container',
                                                   parents=[parent_parser, container_common_parser])
    stop_container_parser = subparsers.add_parser('stop-container', help='Stop a container',
                                                  parents=[parent_parser, container_common_parser])
    terminate_container_parser = subparsers.add_parser('terminate-container', help='Terminate a container',
                                                       parents=[parent_parser, container_common_parser])
    logs_container_parser = subparsers.add_parser('logs-container', help='Get logs from a container',
                                                  parents=[parent_parser, container_common_parser])

    return containers_parser



from tabulate import tabulate
import datetime
import re
import tutum

from tutumcli.exceptions import NonUniqueIdentifier, ObjectNotFound, BadParameter


def tabulate_result(data_list, headers):
    print tabulate(data_list, headers, stralign="left")


def humanize_date_difference_from_now(target_datetime):
    now = datetime.datetime.utcnow()
    dt = target_datetime - datetime.datetime.utcnow()
    offset = dt.seconds + (dt.days * 60*60*24)
    delta_s = offset % 60
    offset /= 60
    delta_m = offset % 60
    offset /= 60
    delta_h = offset % 24
    offset /= 24
    delta_d = offset

    if delta_d > 1:
        if delta_d > 6:
            date = now + datetime.timedelta(days=-delta_d, hours=-delta_h, minutes=-delta_m)
            return date.strftime('%A, %Y %B %m, %H:%I')
        else:
            wday = now + datetime.timedelta(days=-delta_d)
            return wday.strftime('%A')
    if delta_d == 1:
        return "Yesterday"
    if delta_h > 0:
        return "%dh%dm ago" % (delta_h, delta_m)
    if delta_m > 0:
        return "%dm%ds ago" % (delta_m, delta_s)
    else:
        return "%ds ago" % delta_s


def is_uuid4(identifier):
    uuid4_regexp = re.compile('^[a-f0-9]{8}-[a-f0-9]{4}-4[a-f0-9]{3}-[89ab][a-f0-9]{3}-[a-f0-9]{12}', re.I)
    match = uuid4_regexp.match(identifier)
    return bool(match)


def fetch_container(identifier):

    if is_uuid4(identifier):
        return tutum.Container.fetch(identifier)
    else:
        objects_same_identifier = tutum.Container.list(unique_name=identifier) or \
                                  tutum.Container.list(uuid__startswith=identifier)
        if len(objects_same_identifier) == 1:
            return objects_same_identifier[0]
        elif len(objects_same_identifier) == 0:
            raise ObjectNotFound("Cannot find a container with the identifier '%s'" % identifier)
        raise NonUniqueIdentifier("More than one container has the same identifier, please use the long uuid")


def fetch_app(identifier):

    if is_uuid4(identifier):
        return tutum.Application.fetch(identifier)
    else:
        objects_same_identifier = tutum.Application.list(name=identifier) or \
                                  tutum.Application.list(uuid__startswith=identifier)
        if len(objects_same_identifier) == 1:
            return objects_same_identifier[0]
        elif len(objects_same_identifier) == 0:
            raise ObjectNotFound("Cannot find an application with the identifier '%s'" % identifier)
        raise NonUniqueIdentifier("More than one application has the same identifier, please use the long uuid")


def parse_ports(port_list):
    if port_list is not None:
        parsed_ports = []
        for port in port_list:
            parsed_ports.append(_get_port_dict(port))
    else:
        parsed_ports = None
    return parsed_ports


def _get_port_dict(port):
    port_regexp = re.compile('^[0-9]{1,5}/(tcp|udp)$', re.I)
    match = port_regexp.match(port)
    if bool(match):
        port = port.split("/", 1)
        inner_port = int(port[0])
        protocol = port[1].lower()
        return {'protocol': protocol, 'inner_port': inner_port}
    raise BadParameter("Port argument %s does not match with 'port/protocol'. Example: 80/tcp" % port)


def parse_envvars(envvar_list):
    if envvar_list is not None:
        parsed_envvars = []
        for envvar in envvar_list:
            parsed_envvars.append(_is_envvar(envvar))
    else:
        parsed_envvars = None
    return parsed_envvars


def _is_envvar(envvar):
    envvar_regexp = re.compile('^[a-zA-Z_]+[a-zA-Z0-9_]*=[^?!=]+$', re.I)
    match = envvar_regexp.match(envvar)
    if bool(match):
        envvar = envvar.split("=", 1)
        return {'key': envvar[0], 'value': envvar[1]}
    raise BadParameter("Environment Variable argument %s does not match with 'KEY=VALUE'. Example: ENVVAR=foo" % envvar)




from setuptools import setup, find_packages

setup(
    name = "tutum",
    version = "0.6.6",
    packages = find_packages(),
    install_requires = ['python-tutum==0.6.4', 'requests>=2.2.1', 'six==1.6.1', 'tabulate==0.7.2', 'wsgiref==0.1.2'],
    entry_points={
        'console_scripts':
            ['tutum = tutumcli.tutum_cli:main']
        },
    author = "Tutum, Inc.",
    author_email = "info@tutum.co",
    description = "CLI for Tutum",
    license = "Apache v2",
    keywords = "tutum docker cli",
    url = "http://www.tutum.co/",
)